package com.company.sample.view.pointswiss;

import com.company.sample.entity.PointSwiss;
import com.company.sample.view.main.MainView;
import com.vaadin.flow.component.ClickEvent;
import com.vaadin.flow.component.HasValueAndElement;
import com.vaadin.flow.component.formlayout.FormLayout;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.router.Route;
import io.jmix.flowui.kit.action.ActionPerformedEvent;
import io.jmix.flowui.kit.component.button.JmixButton;
import io.jmix.flowui.model.*;
import io.jmix.flowui.view.*;

@Route(value = "pointSwisses", layout = MainView.class)
@ViewController("PointSwiss.list")
@ViewDescriptor("point-swiss-list-view.xml")
@LookupComponent("pointSwissesDataGrid")
@DialogMode(width = "64em")
public class PointSwissListView extends StandardListView<PointSwiss> {

    @ViewComponent
    private DataContext dataContext;

    @ViewComponent
    private CollectionContainer<PointSwiss> pointSwissesDc;

    @ViewComponent
    private InstanceContainer<PointSwiss> pointSwissDc;

    @ViewComponent
    private InstanceLoader<PointSwiss> pointSwissDl;

    @ViewComponent
    private VerticalLayout listLayout;

    @ViewComponent
    private FormLayout form;

    @ViewComponent
    private HorizontalLayout detailActions;

    @Subscribe
    public void onInit(final InitEvent event) {
        updateControls(false);
    }

    @Subscribe("pointSwissesDataGrid.create")
    public void onPointSwissesDataGridCreate(final ActionPerformedEvent event) {
        dataContext.clear();
        PointSwiss entity = dataContext.create(PointSwiss.class);
        pointSwissDc.setItem(entity);
        updateControls(true);
    }

    @Subscribe("pointSwissesDataGrid.edit")
    public void onPointSwissesDataGridEdit(final ActionPerformedEvent event) {
        updateControls(true);
    }

    @Subscribe("saveBtn")
    public void onSaveButtonClick(final ClickEvent<JmixButton> event) {
        dataContext.save();
        pointSwissesDc.replaceItem(pointSwissDc.getItem());
        updateControls(false);
    }

    @Subscribe("cancelBtn")
    public void onCancelButtonClick(final ClickEvent<JmixButton> event) {
        dataContext.clear();
        pointSwissDl.load();
        updateControls(false);
    }

    @Subscribe(id = "pointSwissesDc", target = Target.DATA_CONTAINER)
    public void onPointSwissesDcItemChange(final InstanceContainer.ItemChangeEvent<PointSwiss> event) {
        PointSwiss entity = event.getItem();
        dataContext.clear();
        if (entity != null) {
            pointSwissDl.setEntityId(entity.getId());
            pointSwissDl.load();
        } else {
            pointSwissDl.setEntityId(null);
            pointSwissDc.setItem(null);
        }
    }

    private void updateControls(boolean editing) {
        form.getChildren().forEach(component -> {
            if (component instanceof HasValueAndElement<?, ?> field) {
                field.setReadOnly(!editing);
            }
        });

        detailActions.setVisible(editing);
        listLayout.setEnabled(!editing);
    }
}